a.doesnotexist {
  color: #BA0000;
}

/* Revision comparison form styling */

table.diff {
  margin-top: 2em;
}

table.diff td div {
  height: 2em;
}

table.diff td.rev1 div {
  text-align: right;
}

table.diff td div input {
  margin: 0 10px;
}

table.diff td div.current a {
  font-weight: bold;
}
